@@ -134,9 +134,11 @@ TermDef       FuncTerms[F_FUNCTIONS_NUM + 1] = {
 	FUNC_TERM2 (NEED_NAME, "PopUp", F_POPUP),	/* PopUp    "popup_name" [popup_name] */
 	FUNC_TERM2 (NEED_NAME, "Function", F_FUNCTION),	/* Function "function_name" [function_name] */
 	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Category", F_CATEGORY),	/* Category "function_name" category_name */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "CategoryTree", F_CATEGORY_TREE),	/* CategoryTree "function_name" category_name */
 	FUNC_TERM ("MiniPixmap", F_MINIPIXMAP),	/* MiniPixmap "name" */
 	FUNC_TERM ("SmallMiniPixmap", F_SMALL_MINIPIXMAP),	/* SmallMiniPixmap "name" */
 	FUNC_TERM ("LargeMiniPixmap", F_SMALL_MINIPIXMAP),	/* LargeMiniPixmap "name" */
+	FUNC_TERM2 (NEED_NAME, "DesktopEntry", F_DesktopEntry),	/* DesktopEntry "name" */
 	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Exec", F_EXEC),	/* Exec   "name" command */
 	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Module", F_MODULE),	/* Module "name" command */
     FUNC_TERM2 (NEED_NAME | NEED_CMD, "ExecInTerm", F_ExecInTerm),   /* ExecInTerm   "name" command */
@@ -569,6 +571,7 @@ InitMyApp (  const char *app_class, int argc, char **argv, void (*version_func)
 	}
 
     as_app_args.mask = opt_mask ;
+	as_app_args.gravity = ForgetGravity ;
 #ifndef NO_DEBUG_OUTPUT
     as_app_args.verbosity_level = OUTPUT_VERBOSE_THRESHOLD ;
 #else
@@ -669,6 +672,7 @@ free_func_hash ()
 static char *_as_known_terms[]	= 
 {	
 	"aterm",
+	"x-terminal-emulator",
 	"rxvt  -tr -fg yellow -bg black",
 	"eterm -tr -tint blue -fg yellow -bg black",
 	"xterm -fg yellow -bg blue",
@@ -686,6 +690,7 @@ static char *_as_known_browsers[]	=
 };
 static char *_as_known_editors[]	= 
 {
+	"editor",
 	"$EDITOR",
 	"nedit",
 	"xemacs",
@@ -873,8 +878,8 @@ destroy_asdatabase()
     destroy_user_database();
 }
 
-ASDesktopCategory *
-name2desktop_category( const char *name, ASCategoryTree **tree_return ) 
+static ASCategoryTree*
+name2desktop_category_tree( const char *name, int *tree_name_len ) 
 {
 	ASCategoryTree *ct = CombinedCategories ; 
 	int offset = 0 ;
@@ -901,11 +906,35 @@ name2desktop_category( const char *name, ASCategoryTree **tree_return )
 		ct = CombinedCategories; 
 		offset = 9 ;
 	}
+	if( tree_name_len ) 
+		*tree_name_len = offset ;
+	return ct;
+}
+
+ASDesktopCategory *
+name2desktop_category( const char *name, ASCategoryTree **tree_return ) 
+{
+	int offset = 0 ;
+	ASCategoryTree *ct = name2desktop_category_tree( name, &offset );  
+
 	if( tree_return ) 
 		*tree_return = ct ;
+
 	return fetch_desktop_category( ct, name+offset );
 }
 
+ASDesktopEntry *
+name2desktop_entry( const char *name, ASCategoryTree **tree_return ) 
+{
+	int offset = 0 ;
+	ASCategoryTree *ct = name2desktop_category_tree( name, &offset );  
+
+	if( tree_return ) 
+		*tree_return = ct ;
+
+	return fetch_desktop_entry( ct, name+offset );
+}
+
 void
 InitSession()
 {
